/*
 * Copyright (c) 2023, Cloudera, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

DROP TABLE IF EXISTS HIVE_STATEMENT_${uuid};
DROP TABLE IF EXISTS HIVE_SQL_FILE_${uuid};

CREATE TABLE IF NOT EXISTS HIVE_SQL_FILE_${uuid}
(
    ORIGINAL_PATH        VARCHAR2(MAX),
    PROCESSED_PATH       VARCHAR2(MAX),
    primary key (ORIGINAL_PATH)
)
AS
SELECT ORIGINAL_PATH, PROCESSED_PATH FROM CSVREAD('${path_to_mapping_csv_file}');

MERGE INTO HIVE_SQL_FILE AS F USING (SELECT * FROM HIVE_SQL_FILE_${uuid}) AS H
ON F.SOURCE_SERVER_ID = ${sourceServerId} AND F.SOURCE_CLUSTER_ID = '${clusterName}' AND F.PATH = H.ORIGINAL_PATH
WHEN MATCHED THEN
    UPDATE SET F.WORKING_FILE_PATH = H.PROCESSED_PATH, F.CONTENT = FILE_READ(H.PROCESSED_PATH, 'UTF-8')
WHEN NOT MATCHED THEN
    INSERT (SOURCE_SERVER_ID, SOURCE_CLUSTER_ID, PATH, WORKING_FILE_PATH, CONTENT)
    VALUES (${sourceServerId}, '${clusterName}', H.ORIGINAL_PATH, H.PROCESSED_PATH, FILE_READ(H.PROCESSED_PATH, 'UTF-8'));

CREATE TABLE IF NOT EXISTS HIVE_STATEMENT_${uuid}
(
    PATH                 VARCHAR2(MAX),
    STATEMENT            VARCHAR2(MAX),
    ERROR_MESSAGE        VARCHAR2(MAX),
    primary key (PATH, STATEMENT)
)
AS
SELECT PATH, CASE WHEN STATEMENT IS NULL THEN '' ELSE STATEMENT END AS STATEMENT, ERROR AS ERROR_MESSAGE FROM CSVREAD('${path_to_csv_file}');

MERGE INTO HIVE_STATEMENT AS T USING HIVE_STATEMENT_${uuid} AS S
ON T.SOURCE_SERVER_ID = ${sourceServerId} AND T.SOURCE_CLUSTER_ID = '${clusterName}' AND T.PATH = S.PATH AND T.STATEMENT = S.STATEMENT
WHEN MATCHED THEN
    UPDATE SET T.ERROR_MESSAGE = S.ERROR_MESSAGE
WHEN NOT MATCHED THEN
    INSERT (SOURCE_SERVER_ID, SOURCE_CLUSTER_ID, PATH, STATEMENT, ERROR_MESSAGE)
    VALUES (${sourceServerId}, '${clusterName}', S.PATH, S.STATEMENT, S.ERROR_MESSAGE);


DROP TABLE IF EXISTS HIVE_STATEMENT_${uuid};
DROP TABLE IF EXISTS HIVE_SQL_FILE_${uuid};